; GIF LITE v1.0 by Willow
; Written in pure assembler by Ivushkin Andrey aka Willow
;
; This include file will contain functions to handle GIF image format
;
; Created: August 15, 2004
; Last changed: August 17, 2004

; Change COLOR_ORDER in your program
; if colors are displayed improperly

if ~ (COLOR_ORDER in <MENUETOS,OTHER>)
; This message may not appear under MenuetOS, so watch...
    display 'Please define COLOR_ORDER: MENUETOS or OTHER.',13,10
end if

_null fix 0x1000

; ****************************************
;   FUNCTION ReadGIF - unpacks GIF image
; ****************************************
; in:
;   esi - pointer to GIF file in memory
;   edi - pointer to output raw data
;   eax - pointer to work area (MIN 16 KB!)

; out:
;   eax - 0, all OK;
;   eax - 1, invalid signature;
;   eax - 2, no global color map;
;   eax - 3, raster data not found (yet);
;   eax >=8, unsupported image attributes
;
;   ebx - image width
;   ecx - image height


ReadGIF:
    mov  [.table_ptr],eax
    push edi
    xor  eax,eax
    inc  eax
    cmp  dword[esi],'GIF8'
    jne  .er            ; signature
    mov  ecx,[esi+0xa]
    inc  eax
    btr  ecx,7
    jnc  .er            ; no global color map
    and  ecx,111b
    inc  ecx            ; global color map size
    mov  ebx,1
    shl  ebx,cl
    lea  ebx,[ebx*2+ebx]
    add  esi,0xd
    mov  [.Palette],esi
    lea  edi,[esi+ebx]
    inc  eax
    cmp  byte[edi],0x2c
    jne  .er
    movzx eax,word[edi+5]
    mov  [.img_width],eax
    movzx eax,word[edi+7]
    mov  [.img_height],eax
    movzx eax,word[edi+9]
    cmp  al,8
    jae  .er
    shr  eax,8
    inc  eax
    mov  [.codesize],eax
    lea  esi,[edi+0xb]
    mov  edi,[.table_ptr]
    xor  eax,eax
    cld
    lodsb               ; eax - block_count
    add  eax,esi
    mov  [.block_ofs],eax
    mov  [.bit_count],8
    mov  ecx,[.codesize]
    dec  ecx
    mov  eax,1
    shl  eax,cl
    mov  [.CC],eax
    inc  eax
    mov  [.EOI],eax
    lea  ecx,[eax+1]
    mov  eax, _null shl 16
  .filltable:
    stosd
    inc  eax
    loop .filltable
    pop  edi
  .reinit:
    mov  edx,[.EOI]
    inc  edx
    push [.codesize]
    pop  [.compsize]
    call .Gif_get_sym
    cmp  eax,[.CC]
    je   .reinit
    call .Gif_output
  .cycle:
    movzx ebx,ax
    call .Gif_get_sym
    cmp  eax,edx
    jae  .notintable
    cmp  eax,[.CC]
    je   .reinit
    cmp  eax,[.EOI]
    je   .end
    call .Gif_output
  .add:
    push eax
    mov  eax,[.table_ptr]
    mov  [eax+edx*4],ebx
    pop  eax
    cmp  edx,0xFFF
    jae  .cycle
    inc  edx
    bsr  ebx,edx
    cmp  ebx,[.compsize]
    jne  .noinc
    inc  [.compsize]
  .noinc:
    jmp  .cycle
  .notintable:
    push eax
    mov  eax,ebx
    call .Gif_output
    push ebx
    movzx eax,bx
    call .Gif_output
    pop  ebx eax
    jmp  .add
  .er:
    pop  edi
    jmp  .ex
  .end:
    xor  eax,eax
    mov  ebx,[.img_width]
    mov  ecx,[.img_height]
  .ex:
    ret

.Gif_get_sym:
    mov  ecx,[.compsize]
    push ecx
    xor  eax,eax
  .shift:
    ror  byte[esi],1
    rcr  eax,1
    dec  [.bit_count]
    jnz  .loop1
    inc  esi
    cmp  esi,[.block_ofs]
    jne  .noblock
    push eax
    movzx eax,byte[esi]
    inc  esi
    add  eax,esi
    mov  [.block_ofs],eax
    pop  eax
  .noblock:
    mov  [.bit_count],8
  .loop1:
    loop .shift
    pop  ecx
    rol  eax,cl
    xor  ecx,ecx
    ret

.Gif_output:
    push esi eax edx
    mov  edx,[.table_ptr]
  .next:
    push word[edx+eax*4]
    mov  ax,word[edx+eax*4+2]
    inc  ecx
    cmp  ax,_null
    jnz  .next
    shl  ebx,16
    mov  bx,[esp]
  .loop2:
    pop  ax

    lea  esi,[eax+eax*2]
    add  esi,[.Palette]

    if COLOR_ORDER eq MENUETOS
        mov  esi,[esi]
        bswap esi
        shr  esi,8
        mov  [edi],esi
        add  edi,3
    else
        movsw
        movsb
    end if

    loop .loop2
    pop  edx eax esi
    ret

    .codesize rd 1
    .compsize rd 1
    .bit_count rd 1
    .CC rd 1
    .EOI rd 1
    .Palette rd 1
    .block_ofs rd 1
    .table_ptr rd 1
    .img_width rd 1
    .img_height rd 1
